Encoding: UTF-8
Type: Package
Package: mr.mashr
Version: 0.3.44
Date: 2025-12-21
Title: Multiple Regression with Multivariate Adaptive Shrinkage
Description: Provides an implementation of methods for multivariate 
    multiple regression with adaptive shrinkage priors as
    described in F. Morgante et al (2023) <doi:10.1371/journal.pgen.1010539>.
URL: https://github.com/stephenslab/mr.mashr
Authors@R: c(person("Fabio","Morgante",role="aut",
                    email="fabiom@clemson.edu"),
             person("Jason","Willwerscheid",role="ctb"),
             person("Gao","Wang",role="ctb"),
             person("Deborah","Kunkel",role="aut"),
             person("Daniel","Nachun",role="ctb"),
             person("Peter","Carbonetto",role=c("cre","aut"),
                    email="peter.carbonetto@gmail.com"),
             person("Matthew","Stephens",role="aut"))
License: MIT + file LICENSE
Depends: R (>= 3.1.0)
SystemRequirements: GNU make
Imports: methods, stats, Matrix, Rcpp (>= 1.1.0), RcppParallel (>=
        5.1.10), mvtnorm, matrixStats, mashr (>= 0.2.73), ebnm,
        flashier (>= 1.0.7), parallel
Suggests: testthat, varbvs, knitr, rmarkdown,
LinkingTo: Rcpp, RcppArmadillo (>= 0.10.4.0.0), RcppParallel
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2025-12-21 17:56:12 UTC; pcarbo
Author: Fabio Morgante [aut],
  Jason Willwerscheid [ctb],
  Gao Wang [ctb],
  Deborah Kunkel [aut],
  Daniel Nachun [ctb],
  Peter Carbonetto [cre, aut],
  Matthew Stephens [aut]
Maintainer: Peter Carbonetto <peter.carbonetto@gmail.com>
Repository: CRAN
Date/Publication: 2026-01-07 08:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 19:56:00 UTC; unix
Archs: mr.mashr.so.dSYM
