# Generated by roxygen2: do not edit by hand

S3method(coef,mr.mash)
S3method(coef,mr.mash.rss)
S3method(predict,mr.mash)
S3method(predict,mr.mash.rss)
export(autoselect.mixsd)
export(coef.mr.mash)
export(coef.mr.mash.rss)
export(compute_canonical_covs)
export(compute_data_driven_covs)
export(compute_univariate_sumstats)
export(expand_covs)
export(mr.mash)
export(mr.mash.rss)
export(predict.mr.mash)
export(predict.mr.mash.rss)
export(simulate_mr_mash_data)
importFrom(Matrix,crossprod)
importFrom(Matrix,diag)
importFrom(Matrix,isSymmetric)
importFrom(Matrix,t)
importFrom(Rcpp,evalCpp)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(RcppParallel,defaultNumThreads)
importFrom(RcppParallel,setThreadOptions)
importFrom(ebnm,ebnm_normal)
importFrom(ebnm,ebnm_normal_scale_mixture)
importFrom(flashier,flash)
importFrom(mashr,cov_ed)
importFrom(mashr,cov_flash)
importFrom(mashr,cov_pca)
importFrom(mashr,get_significant_results)
importFrom(mashr,mash_1by1)
importFrom(mashr,mash_set_data)
importFrom(matrixStats,colMeans2)
importFrom(matrixStats,colSds)
importFrom(matrixStats,colVars)
importFrom(methods,as)
importFrom(mvtnorm,dmvnorm)
importFrom(mvtnorm,rmvnorm)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(stats,coef)
importFrom(stats,cov)
importFrom(stats,cov2cor)
importFrom(stats,lm)
importFrom(stats,predict)
importFrom(stats,rnorm)
useDynLib(mr.mashr)
